package operators;
import informationexchange.LogService;
import knowledge.*;
import java.util.Iterator;
import exceptions.*;
import java.util.LinkedList;
import dataanalysis.tools.StringMatcher;

/**
   automatisch erzeugter OperatorCompageContainsKeywordOperator2
   @author Oliver Geppert
   @version [1999-12-14]
*/
public class CompageContainsKeywordOperator2 extends Operator{

  /**
    Automatisch generierter Konstruktor fr den 
    CompageContainsKeywordOperator2.
        Vorbedingung: keyword(key)
                      has_compage(comname,compage)
    1. Nachbedingung: *Leer*
    2. Nachbedingung: contains_keyword(compage,key)
    @since 1999-12-14
   */

  public CompageContainsKeywordOperator2(){
    /* 2 Vorbedingung(en) */
    this.precond=new Predicate[2];
    /* Vorbedindungsprdikat 0 */
    this.precond[0]=new Predicate(1,"keyword");

    PredArgument key = new PredArgument("key");
    key.setCreatesWorldObject(false);
    this.precond[0].setPredArg(key, 0);
    /* Vorbedindungsprdikat 1 */
    this.precond[1]=new Predicate(2,"has_compage");

    PredArgument comname = new PredArgument("comname");
    comname.setCreatesWorldObject(false);
    this.precond[1].setPredArg(comname, 0);
    PredArgument compage = new PredArgument("compage");
    compage.setCreatesWorldObject(false);
    this.precond[1].setPredArg(compage, 1);

    Predicate pred;
    /* 2 Nachbedingung(en) */
    this.postconds=new Postcondition[2];

    /* Nachbedingung 0 */
    this.postconds[0]=new Postcondition();
    /* AddList */
    /* DeleteList */
    /* Nachbedingung 1 */
    this.postconds[1]=new Postcondition();
    /* AddList */
    pred=new Predicate(2,"contains_keyword");
    pred.setPredArg(compage, 0);
    pred.setPredArg(key, 1);


    this.postconds[1].addAddListPred(pred);

    /* DeleteList */
  }

  /**
     Eval-Methode des CompageContainsKeywordOperators Nr. 2.
     Naeheres siehe {@link operators.Operator#eval()}.
     Die eval-Methode hat die Aufgabe, zu berprfen, ob ein gegebenes
     Keyword auf einer Compage vorkommt. Als Heuristik wird angenommen, da
     es hinreichend ist, wenn das Keyword im Namen der Firma vorkommt.
     @since 1999-12-14
  */
  public void eval(){
    postcondFullfilled = 0;

    try {
      String key = precond[0].getPredArg(0).getInstance().name;
      String comname = precond[1].getPredArg(0).getInstance().name;
      String cptext = precond[1].getPredArg(1).getInstance().
                      getNetEntity().getText();
      Predicate contains_keyword = (Predicate) getPostcond()[1].getAddList().get(0);
    
      comname.toLowerCase();
      key.toLowerCase();

      LogService.logInfo (this, "gesuchtes Keyword: " + key);

      StringMatcher sm = new StringMatcher (key);
      if (sm.fastMatch(cptext) > 0 || comname.indexOf(key) > -1) {
        postcondFullfilled = 1;
        contains_keyword.setConfidence (1.0);
      }

    } catch (NetResultReloadFailedException e) {
      LogService.logException (this, "NetResultReloadFailedException");
      postcondFullfilled = 0;
    } catch (NullPointerException e) {
      LogService.logException (this, "Oeps, wij hadden een algemeene nullwijzerexceptie!");
      postcondFullfilled = 0;
    }
  }
}