package operators;
import dataprovider.enumerationobjects.*;
import dataprovider.net.*;
import exceptions.*;
import informationexchange.LogService;
import knowledge.*;
import dataanalysis.tools.StringMatcher;
import java.util.Arrays;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.ListIterator;

/**
<code>
#------------------------------------------------------------------
# Suche von einer Compage aus eine Prodpage
# Fuer das Produkt productname.
# gehe dabei nicht tiefer als 2 Linkebenen nach unten
# und bleibe auf demselben Server.
# 
#------------------------------------------------------------------
@name FindProdPageFromHomeOperator
@precondition has_compage(comname,compage)
@precondition productname(prodname)
@postcondition
@addlistitem has_productpage(comname,prodpage):n,y
@addlistitem contains_productname(prodpage,prodname):y,n
</code>
   @author Nils Malzahn
   @version 1.0
*/
public class FindProdPageFromHomeOperator extends Operator{
    /**
       maximale LinkTiefe
     */
    static protected int linkdepth=2;

   /**
	Server, der beruecksichtigt werden soll. Alle anderen
	Links werden beim Spidern verworfen.
   */	
	String server=null;
   
    /**
       Produktname, der gefunden werden muss. Alle Seiten, die
       den Produktnamen weder in der URL noch in der Seite selbst
       enthalten werden verworfen.
    */
    String productname=null;

    protected LinkedList spider(HTMLTextNetResult netresult, Concept concept,int depth){
	LinkedList ll=new LinkedList();
	final StringMatcher matcher = new StringMatcher(productname);
	matcher.setIsCaseSensitive(false);
	matcher.setMustFindAll(true);
	matcher.setDoPhraseSearch(true);

	if (!(depth<linkdepth)) return (ll);
	/* Maximale Suchtiefe noch nicht erreicht */
	try{
	    HTMLLinkEnumeration linkEnumeration=netresult.getLinks();
	    while(linkEnumeration.hasMoreElements()){
		try{
		    Link result = (Link) linkEnumeration.nextElement();
		    if (result.getURL().indexOf(server)>-1){
		      LogService.logOperator( this, "trying to download " + result.getURL() );
		      NetService ns=NetService.getNetService(result.getURL());
		    
		      HTMLTextNetResult htmlresult=(HTMLTextNetResult) ns.getNetResult(result.getURL());
		    
		      // Nur zurueckgeben, wenn Produktname im Text oder URL gefunden wurde...
		      try{
			  if( (matcher.fastMatch(htmlresult.getURL())>-1)||
			      (matcher.fastMatch(htmlresult.getText())>-1)){
			      Instance aktInst = new Instance(htmlresult,concept);
			      if (concept.classify(aktInst)){
				  ll.add(aktInst);
			      };
			  };
		      }catch(NetResultReloadFailedException e1){
			  LogService.logOperator(this,"Exception "+e1+ "occured");
		      };
		      /* Teilergebnisse erzeugen und aufsammeln */
		      ListIterator li=spider(htmlresult,concept,depth+1).listIterator();
		      while(li.hasNext()) ll.add(li.next());
		    }
		}
		catch(ClassCastException e1){
		    LogService.logOperator(this,"Exception "+e1+ "occured");
		}
		catch (NetServiceException e2){
		    LogService.logOperator(this,"Exception "+e2+ "occured");
		}
		catch (java.net.MalformedURLException e3){
		    LogService.logOperator(this,"Exception "+e3+ "occured");
		};
	    }
	}
	
	catch(NetResultReloadFailedException e4){
	    return(ll);
	};
	return(ll);
    }
    

    /**
       Eval-Methode des Operators.
       Naeheres siehe {@link operators.Operator#eval()}.
    */
    public void eval() throws OperatorFailedException{
	LinkedList possibleResult=null;
	final Predicate has_productpage = ((Predicate)postconds[0].getAddList().getFirst());
	final Predicate contains_productname = ((Predicate)postconds[0].getAddList().getLast());
	PredArgument prodpagePredArg= has_productpage.getPredArg(1);
	final Concept prodpageconcept = prodpagePredArg.getConcept();
	System.out.println("Start eval");
	productname = precond[1].getPredArg(0).getInstance().name;
	System.out.println("productname = " + productname);

	try{
	    final HTMLTextNetResult compage = (HTMLTextNetResult) precond[0].getPredArg(1).getInstance().getNetEntity();
	    System.out.println("Instance = " + precond[0].getPredArg(1).getInstance());
	    System.out.println("compage = " + compage);
	    server=compage.getURL();
	    System.out.println("server = " + server);
		int doubleslash=server.indexOf("//")+2;
	    System.out.println("doubleslash = " + doubleslash);
	    server=server.substring(doubleslash,server.indexOf("/",doubleslash));
	    System.out.println("server = " + server);
	    possibleResult=spider(compage,prodpageconcept,0);
	    System.out.println("possibleResult = " + possibleResult);
	}
	catch (ClassCastException cce){
	    throw new OperatorFailedException("compage was not a HTMLTextNetResult!");
	}
	System.out.println("after try: possibleResult = "+ possibleResult);
	if (possibleResult==null) throw new OperatorFailedException("Didn't find a Productpage!");
	Object[] possibleResultArray=possibleResult.toArray();
	if( possibleResult.size() > 1 )
			Arrays.sort(possibleResultArray,new Comparator(){
				public int compare(Object o1, Object o2){
				    Instance inst1=(Instance) o1;
				    Instance inst2=(Instance) o2;
				    return((new Double(inst1.getConfidence(prodpageconcept))).compareTo(new Double(inst2.getConfidence(prodpageconcept))));
					
				}
			    });
	possibleResult=new LinkedList(Arrays.asList(possibleResultArray));
	if( possibleResult.size() > 0 )
	{
		Instance resultInstance=(Instance)possibleResult.getLast();
		prodpagePredArg.setInstance(resultInstance);
		/*
		  P(has_productpage|has_compage)=P(prodpage)*P(has_compage)
		 */
		has_productpage.setConfidence(precond[0].getConfidence()*resultInstance.getConfidence(prodpageconcept));
		LogService.logOperator( this,resultInstance.name + 
					", P(has_compage)=" + 
					precond[0].getConfidence() +
					", P(prodpage)=" +
					resultInstance.getConfidence(prodpageconcept));
		 LogService.logOperator(this,"has_compage_role_cofidence: "+precond[0].getPredArg(0).getInstance().getABox().getConcreteRole(precond[0].getPredArg(0).getConcept().getRole(precond[0].getName()),precond[0].getPredArg(0).getInstance(),precond[0].getPredArg(1).getInstance()).getConfidence()); 
		contains_productname.setConfidence(1.0);
		possibleResult.remove(possibleResult.getLast());
		ListIterator li=possibleResult.listIterator();
		while(li.hasNext()){
		    prodpagePredArg.addInstance((Instance) li.next());
		}
	}
	else throw new OperatorFailedException("There was no SolutionInstance");
		

	
	this.postcondFullfilled=0;
    }
    
  public FindProdPageFromHomeOperator(){
    /* 2 Vorbedingung(en) */
    this.precond=new Predicate[2];
    /* Vorbedindungsprdikat 0 */
    this.precond[0]=new Predicate(2,"has_compage");

    PredArgument comname = new PredArgument("comname");
    comname.setCreatesWorldObject(false);
    this.precond[0].setPredArg(comname, 0);
    PredArgument compage = new PredArgument("compage");
    compage.setCreatesWorldObject(false);
    this.precond[0].setPredArg(compage, 1);

    /* Vorbedingungsprdikat 1 */
    this.precond[1]=new Predicate(1,"productname");

    PredArgument prodname = new PredArgument("prodname");
    prodname.setCreatesWorldObject(false);
    this.precond[1].setPredArg(prodname,0);


    Predicate pred;
    /* 1 Nachbedingung(en) */
    this.postconds=new Postcondition[1];

    /* Nachbedingung 0 */
    this.postconds[0]=new Postcondition();
    /* AddList */
    pred=new Predicate(2,"has_productpage");
    pred.setPredArg(comname, 0);
    PredArgument prodpage = new PredArgument("prodpage");
    prodpage.setCreatesWorldObject(true);
    pred.setPredArg(prodpage, 1);

    this.postconds[0].addAddListPred(pred);

    pred=new Predicate(2,"contains_productname");
    pred.setPredArg(prodpage,0);
    pred.setPredArg(prodname,1);

    this.postconds[0].addAddListPred(pred);

    /* DeleteList */
  }
}


