package operators;
import informationexchange.LogService;
import knowledge.*;
import java.util.*;
import exceptions.*;
import java.util.LinkedList;
import dataprovider.net.*;
import dataprovider.search.*;
import dataprovider.enumerationobjects.*;
import dataanalysis.tools.StringMatcher;


/**
   automatisch erzeugter OperatorFindProdPageOperator
<code>
------------------------------------------------------------------
 (a) Suche prodpage mit einem search service. In die Suchanfrage
     geht mind. companyname, productname & ein trade-synonym ein
 (b) Verifiziere die Ergebnisse nach prodpage mit Klassifikator
------------------------------------------------------------------
@name FindProdPageOperator
@precondition in_trade(comname,trade)
@precondition deals_in(comname,prodname)
@precondition relates_to(synonym,trade)
@postcondition
@addlistitem has_productpage(comname,prodpage):n,y
@addlistitem contains_productname(prodpage,prodname)y,n
</code>
   @author Markus H&ouml;vener
   @version 1.0
*/
public class FindProdPageOperator extends Operator
{
	public SearchService  service;


	/**
	   Eval-Methode des Operators.
	   Naeheres siehe {@link operators.Operator#eval()}.
	*/
	public void eval() throws OperatorFailedException
	{
		SearchNetResult          results;
		Enumeration              enumeration;
		SearchServiceResult      result;
		final Instance           comname_instance = this.precond[0].getPredArg(0).getInstance();
		final Instance           synonym_instance = this.precond[2].getPredArg(0).getInstance();
		final Instance           trade_instance = this.precond[0].getPredArg(1).getInstance();
		final Instance           prodname_instance = this.precond[1].getPredArg(1).getInstance();
		PredArgument             compage_pred_arg = ((Predicate)this.postconds[0].getAddList().getFirst()).getPredArg(1);
		double                   compage_pred_arg_confidence = 0.0;
		final Concept            compage_concept = compage_pred_arg.getConcept();
		ListIterator             li;
		NetService               net_service;
		LinkedList               list = new LinkedList();
		NetResult                net_result;
		Predicate                has_productpage_predicate = (Predicate) (this.getPostcond())[0].getAddList().getFirst();
		Predicate                contains_productname_predicate = (Predicate) this.postconds[0].getAddList().getLast();
		Instance                 curr_instance;
		String                   search_string;
		int                      folder_index = YahooSearchService.FolderNone;
		StringMatcher            matcher=new StringMatcher( prodname_instance.name );

		matcher.setIsCaseSensitive(false);
		matcher.setMustFindAll(true);

		LogService.logOperator( this, "FindProdPageOperator( comname = " + comname_instance.name + ", synonym = " + synonym_instance.name + ", trade = " + trade_instance.name + ", prodname = " + prodname_instance.name  + ")" );

		search_string = comname_instance.name + " " + synonym_instance.name;

		if( service instanceof YahooSearchService )
		{
			if( trade_instance.name.equals( "landwirtschaft" ) )  folder_index = YahooSearchService.FolderAgriculture;
			if( trade_instance.name.equals( "antiquitaeten" ) )  folder_index = YahooSearchService.FolderAntiqueAndCollect;
			if( trade_instance.name.equals( "computer" ) )  folder_index = YahooSearchService.FolderComputer;
			if( trade_instance.name.equals( "konferenzen+messen" ) )  folder_index = YahooSearchService.FolderConferenceAndFair;
			if( trade_instance.name.equals( "bauwesen" ) )  folder_index = YahooSearchService.FolderConstructoral;
			if( trade_instance.name.equals( "essen+trinken" ) )  folder_index = YahooSearchService.FolderEatAndDrink;
			if( trade_instance.name.equals( "umwelt" ) )  folder_index = YahooSearchService.FolderEnvironment;
			if( trade_instance.name.equals( "information" ) )  folder_index = YahooSearchService.FolderInformation;
			if( trade_instance.name.equals( "marketing" ) )  folder_index = YahooSearchService.FolderMarketing;
			if( trade_instance.name.equals( "nachrichten+medien" ) )  folder_index = YahooSearchService.FolderNewsAndMedia;
			if( trade_instance.name.equals( "outdoor" ) )  folder_index = YahooSearchService.FolderOutdoor;
			if( trade_instance.name.equals( "verpackungen" ) )  folder_index = YahooSearchService.FolderPacking;
			if( trade_instance.name.equals( "fotografie" ) )  folder_index = YahooSearchService.FolderPhotography;
			if( trade_instance.name.equals( "politik+verwaltung" ) )  folder_index = YahooSearchService.FolderPoliticAndAdministration;
			if( trade_instance.name.equals( "geschenke" ) )  folder_index = YahooSearchService.FolderPresents;
			if( trade_instance.name.equals( "drucken" ) )  folder_index = YahooSearchService.FolderPrinting;
			if( trade_instance.name.equals( "wissenschaft" ) )  folder_index = YahooSearchService.FolderScience;
			if( trade_instance.name.equals( "telekommunikation" ) )  folder_index = YahooSearchService.FolderTelecommunication;
			if( trade_instance.name.equals( "spielzeug" ) )  folder_index = YahooSearchService.FolderToys;
			if( trade_instance.name.equals( "reisen" ) )  folder_index = YahooSearchService.FolderTraveling;

			if( folder_index != YahooSearchService.FolderNone )
				LogService.logOperator( this, "Yahoo search is limited to folder #" + folder_index );
		}


		try
		{
			LogService.logOperator( this, "Submitting query (" + search_string + ") to search engine" );
			if( folder_index != YahooSearchService.FolderNone )
				results = ((YahooSearchService)service).doQuery( search_string, folder_index );
			else
				results = service.doQuery( search_string );
			LogService.logOperator( this, "Now checking search engine results" );


			enumeration = results.getResults();

			while( enumeration.hasMoreElements() )
			{
				result = (SearchServiceResult)enumeration.nextElement();

				LogService.logOperator( this, "Downloading " + result.getURL() );


				try
				{
					net_service = NetService.getNetService( result.getURL() );
					net_result  = net_service.getNetResult( result.getURL() );

					curr_instance = new Instance( net_result, compage_concept );


					if( compage_concept.classify( curr_instance ) )
					{
						if( matcher.matchesTo( curr_instance.getNetEntity().getText()) > 0.0 )
							list.add( curr_instance );
						else
							LogService.logOperator( this, "Downloading succeeded, classify() returned true, but prodname not found" );
					}
					else
						LogService.logOperator( this, "Downloading succeeded, but classify() returned false" );
				}
				
				catch( Exception e )
				{
					LogService.logOperator( this, "Downloading failed due to exception " + e.toString() );
				}
			}
		}

		catch( Exception e )
		{
			LogService.logOperator(this, "Exception: " + e.toString() );
			throw new OperatorFailedException();
		}
		

		LogService.logOperator( this, "FindProdPageOperator found " + list.size() + " possible results" );
		Object[] array=list.toArray();
		if( list.size() > 1 )
			Arrays.sort(array,new Comparator()
			{
				public int compare(Object o1, Object o2)
				{
					Instance inst1=(Instance) o1;
					Instance inst2=(Instance) o2;
					return((new Double(inst1.getConfidence(compage_concept))).compareTo(new Double(inst2.getConfidence(compage_concept))));
				}
			});
		list=new LinkedList(Arrays.asList(array));

		if( list.size() == 0 )
			throw new OperatorFailedException();

		compage_pred_arg.setInstance((Instance)list.getLast());
		compage_pred_arg_confidence = compage_pred_arg.getInstance().getConfidence(compage_concept);
		
		list.remove(list.getLast());
		li = list.listIterator();
		while(li.hasNext())
			compage_pred_arg.addInstance((Instance) li.next());

		has_productpage_predicate.setConfidence( compage_pred_arg_confidence );
		contains_productname_predicate.setConfidence( 1.0 );
		this.postcondFullfilled=0;
	}

  public FindProdPageOperator(){/* 3 Vorbedingung(en) */
    this.precond=new Predicate[3];
    /* Vorbedindungsprdikat 0 */
    this.precond[0]=new Predicate(2,"in_trade");

    PredArgument comname = new PredArgument("comname");
    comname.setCreatesWorldObject(false);
    this.precond[0].setPredArg(comname, 0);
    PredArgument trade = new PredArgument("trade");
    trade.setCreatesWorldObject(false);
    this.precond[0].setPredArg(trade, 1);

    /* Vorbedindungsprdikat 1 */
    this.precond[1]=new Predicate(2,"deals_in");

    this.precond[1].setPredArg(comname, 0);
    PredArgument prodname = new PredArgument("prodname");
    prodname.setCreatesWorldObject(false);
    this.precond[1].setPredArg(prodname, 1);

    /* Vorbedindungsprdikat 2 */
    this.precond[2]=new Predicate(2,"relates_to");

    PredArgument synonym = new PredArgument("synonym");
    synonym.setCreatesWorldObject(false);
    this.precond[2].setPredArg(synonym, 0);
    this.precond[2].setPredArg(trade, 1);

    Predicate pred;
    /* 1 Nachbedingung(en) */
    this.postconds=new Postcondition[1];

    /* Nachbedingung 0 */
    this.postconds[0]=new Postcondition();
    /* AddList */
    pred=new Predicate(2,"has_productpage");
    pred.setPredArg(comname, 0);
    PredArgument prodpage = new PredArgument("prodpage");
    prodpage.setCreatesWorldObject(true);
    pred.setPredArg(prodpage, 1);


    this.postconds[0].addAddListPred(pred);

    pred=new Predicate(2,"contains_productname");
    prodpage.setCreatesWorldObject(true);
    pred.setPredArg(prodpage, 0);
    pred.setPredArg(prodname, 1);


    this.postconds[0].addAddListPred(pred);

    /* DeleteList */
  }
}
