package operators;
import dataprovider.net.*;
import dataprovider.enumerationobjects.*;
import informationexchange.LogService;
import knowledge.*;
import java.util.Iterator;
import exceptions.*;
import java.util.LinkedList;

/**
   automatisch erzeugter OperatorGuessCompageFromProdpageOperator
   <code>
   @name GuessCompageFromProdpageOperator
   @precondition trade(trade)
   @precondition has_productpage(comname,prodpage)
   @postcondition
   @postcondition
   @addlistitem has_compage(comname,compage):n,y
   @addlistitem in_trade(comname,trade):n,n
   </code>
   @author Nils Malzahn
   @version 1.1
*/
public class GuessCompageFromProdpageCutOperator extends Operator{


    /**
       Hier wird die Compage-URL durch abschneiden der Produktpage-URL geraten.
       @return geratene URL
       @since 1.0
     */
    protected String guessURL(){
	String prodpageURL=precond[1].getPredArg(1).getInstance().getNetEntity().getURL();
	String result=prodpageURL.substring(0,prodpageURL.indexOf("/",prodpageURL.indexOf("//")+2));
	LogService.logOperator(this,"Guessed "+result+" from "+prodpageURL);
	return(result);
    }

/**
       Eval-Methode des Operators.
       Naeheres siehe {@link operators.Operator#eval()}.
    */
    public void eval() throws OperatorFailedException {

      //die URL raten
      String guessedURL = guessURL();
    
      //LogService.logWarning(this,this.getClass()+": eval called().");

      //HTTPNetService erzeugen
      HTTPNetService hns = new HTTPNetService();
      NetResult nr = null; 

      //versuche, die geratene URL per HTTPNetService zu holen
      try {
          nr = hns.getNetResult(guessedURL); 
      }
      catch (java.net.MalformedURLException e) {
          throw new OperatorFailedException("MalformedURLException: Der Operator hat die mailformed URL "+guessedURL+
                                            " geraten :(");
      }
      catch (NetServiceException e) {
          throw new OperatorFailedException("NetServiceException: Konnte geratene URL "+guessedURL+" nicht "+
                                            "holen.");
      }

      LogService.logPlan(this, "Geratene URL "+guessedURL+" konnte geholt werden.");      

      //Erzeuge potentielle compage-Instanz
      Instance nri = new Instance(guessedURL);
      nri.setNetEntity(nr);
    
      //Das spezifische tradepage-Unterkonzept zur trade-Instanz holen
      PredArgument tradePA = this.precond[0].getPredArg(0);
      String tradeAsString = tradePA.getInstance().name;
      T_Box tbox = tradePA.getConcept().getT_Box();
      Concept specTradeC = tbox.lookup(tradeAsString);
    
      //potentielle compage-Instanz bzgl. spezifiscjen tradepage-Unterkonzept klassifizieren
      if(specTradeC.classify(nri)) {
          //...positive Klassifikation
          LogService.logPlan(this, "Positive Klassifikation von "+guessedURL+" nach "+specTradeC.name+" => postcondFullfilled=1.");
	  Predicate postcondPred =  ((Predicate) (this.getPostcond())[1].getAddList().getFirst());
          PredArgument compagePA = postcondPred.getPredArg(1);
          compagePA.setInstance(nri);     
	  /*
	    P(has_compage|has_productpage)=0.9*P(has_productpage)
	   */
	  postcondPred.setConfidence(0.9*precond[1].getConfidence());
          this.postcondFullfilled=1;

      } else {
          //...negative Klassifikation   
          LogService.logPlan(this, "Negative Klassifikation von "+guessedURL+" nach "+specTradeC.name+" => postcondFullfilled=0.");

          this.postcondFullfilled=0;
	  //throw new OperatorFailedException("Exception: classification for concept "+
          //                                  specTradeC.name+" was negative.");

      };//else

  }//eval

  public GuessCompageFromProdpageCutOperator(){
    /* 2 Vorbedingung(en) */
    this.precond=new Predicate[2];
    /* Vorbedindungsprdikat 0 */
    this.precond[0]=new Predicate(1,"trade");

    PredArgument trade = new PredArgument("trade");
    trade.setCreatesWorldObject(false);
    this.precond[0].setPredArg(trade, 0);

    /* Vorbedindungsprdikat 1 */
    this.precond[1]=new Predicate(2,"has_productpage");
    PredArgument comname = new PredArgument("companyname");
    comname.setCreatesWorldObject(false);
    this.precond[1].setPredArg(comname, 0);
    
    PredArgument prodpage = new PredArgument("prodpage");
    prodpage.setCreatesWorldObject(false);
    this.precond[1].setPredArg(prodpage, 1);

    Predicate pred;
    /* 2 Nachbedingung(en) */
    this.postconds=new Postcondition[2];
    
    
    /* Nachbedingung 0 */
    this.postconds[0]=new Postcondition();
    
    /* Nachbedingung 1 */
    this.postconds[1]=new Postcondition();
    /* AddList */
    pred=new Predicate(2,"has_compage");
    pred.setPredArg(comname, 0);
    PredArgument compage = new PredArgument("compage");
    compage.setCreatesWorldObject(true);
    pred.setPredArg(compage, 1);
    this.postconds[1].addAddListPred(pred);

    pred=new Predicate (2,"in_trade");
    pred.setPredArg(comname,0);
    pred.setPredArg(trade,1);
    this.postconds[1].addAddListPred(pred);

    /* DeleteList */
  }
}




