package operators;

import dataanalysis.tools.StringMatcher;
import dataprovider.net.*;
import dataprovider.enumerationobjects.*;
import exceptions.*;
import informationexchange.LogService;
import knowledge.*;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;

/**

   <code>
   @name GuessCompageFromProdpageHomeOperator
   @precondition trade(trade)
   @precondition has_productpage(comname,prodpage)
   @postcondition
   @postcondition
   @addlistitem has_compage(comname,compage):n,y
   @addlistitem in_trade(comname,trade):n,n
   </code>
   @author Nils Malzahn
   @version 1.0
*/
public class GuessCompageFromProdpageHomeOperator extends GuessCompageFromProdpageCutOperator{
    
    /**
       Die Begriffe, die in Linkbeschreibungen zu [Home]-Links vorkommen.
       @since 1.0
     */
    protected transient static final String linkdescrwords="home Leitseite Hauptseite Anfang Startseite Homepage";

    /**
       Die Begriffe, die in LinkURLs vorkommen.
       @since 1.0
     */
    protected transient static final String linkURLwords="home.htm index.htm start.htm";

    /**
       <ol>
       <li>vorrangige Heuristik suchen nach [Home]-Links </li>
       <li>(nachgeordnete )Heuristik suchen nach index.htm* home.htm start.htm</li>
       <li>letzte Heuristik - Heuristik des Verzweifelten:  trifft keine der o.g. Heuristiken zu,
       dann wird behauptet es handelt sich bei der Produktpage auch um eine Firmenseite.</li>
       </ol>
       @return geratene URL
       @since 1.0
     */
    protected String guessURL(){
	NetEntity prodpageNetEntity=precond[1].getPredArg(1).getInstance().getNetEntity();
	/* Es muss ein HTML-Text-NetResult sein, sonst hat dieser Operator keine Chance */
	if (!(prodpageNetEntity instanceof HTMLTextNetResult)) return(null);
	Enumeration resultEnum=null;
	HTMLTextNetResult prodPageNetResult=(HTMLTextNetResult)prodpageNetEntity;
	try {
	    resultEnum=prodPageNetResult.getLinks();
	}
	catch(NetResultReloadFailedException e){
	    /* Wenn die Seite nicht geladen werden kann, dann ist halt nix mit raten */
	    return(null);
	};
	Link link;
	StringMatcher matcher=new StringMatcher(linkdescrwords);
	matcher.setMustFindAll(false);
	matcher.setIsCaseSensitive(false);
	matcher.setDoPhraseSearch(false);
	/* 1. und vorrangige Heuristik suchen nach [Home]-Links */
	LogService.logOperator(this,"using 1st level heuristics");
	while (resultEnum.hasMoreElements()){
	    link=(Link)resultEnum.nextElement();
	    if (matcher.matchesTo(link.getLinkText())>0.0) return(link.getURL());	    
	};

	/* 2. (nachgeordnete )Heuristik suchen nach index.htm* home.htm start.htm */
	LogService.logOperator(this,"using 2nd level heuristics");
	matcher.setPattern(linkURLwords);
	while (resultEnum.hasMoreElements()){
	    link=(Link)resultEnum.nextElement();
	    if (matcher.matchesTo(link.getLinkText())>0.0) return(link.getURL());	    
	    
	};
	LogService.logOperator(this,"using 3rd level heuristics");
	/* 3. und letzte Heuristik - Heuristik des Verzweifelten 
	   Wird keine gefunden, dann wird behauptet es handelt sich
	   bei der Produktpage auch um eine Firmenseite. */
	return(prodPageNetResult.getURL());
    }

  public GuessCompageFromProdpageHomeOperator(){
      super();
  }
}
