package operators;
import informationexchange.LogService;
import knowledge.*;
import java.util.Iterator;
import exceptions.*;
import java.util.LinkedList;
import dataprovider.net.HTTPNetService;
import dataprovider.net.NetResult;


/**
   OperatorGuessCompageOperator (Skelett automatisch erzeugt). 
   Abstrakte Klasse fuer Operatoren, die versuchen, die compage zu raten.
   Konkrete Operatoren raten z.B. bzgl. -.de oder -.com domain. 
   @author Christian Fischbach
   @version 1.0
*/
public abstract class GuessCompageOperator extends Operator{

    /**
       Eval-Methode des Operators.
       Naeheres siehe {@link operators.Operator#eval()}.
    */
    public void eval() throws OperatorFailedException {

      //die URL raten
      String guessedURL = guessURL();

      // eventuell Umlaute etc. umwandeln
      StringBuffer newURL = new StringBuffer();
      int currentpos = 0;
      char currentchar;

      while((guessedURL!=null)&&(currentpos<guessedURL.length())){
	  currentchar = guessedURL.charAt(currentpos);
	  switch(currentchar){
	  case '':
	      newURL.append("ss");
	      break;
	  case '':
	  case '':
	  case '':
	  case '':
	      newURL.append("a");
	      break;
	  case '':
	  case '':
	  case '':
	  case '':
	      newURL.append("e");
	      break;
	  case '':
	  case '':
	  case '':
	  case '':
	      newURL.append("o");
	      break;
	  case '':
	  case '':
	  case '':
	  case '':
	      newURL.append("u");
	      break;
	  case '':
	  case '':
	      newURL.append("ae");
	      break;
	  case '':
	  case '':
	      newURL.append("e");
	      break;
	  case '':
	  case '':
	      newURL.append("oe");
	      break;
	  case '':
	  case '':
	      newURL.append("ue");
	      break;
	  default:
	      newURL.append(currentchar);
	  };
	  currentpos++;
      };
      guessedURL = newURL.toString();

      //LogService.logWarning(this,this.getClass()+": eval called().");

      //HTTPNetService erzeugen
      HTTPNetService hns = new HTTPNetService();
      NetResult nr = null; 

      //versuche, die geratene URL per HTTPNetService zu holen
      try {
          nr = hns.getNetResult(guessedURL); 
      }
      catch (java.net.MalformedURLException e) {
          throw new OperatorFailedException("MalformedURLException: Der Operator hat die mailformed URL "+guessedURL+
                                            " geraten :(");
      }
      catch (NetServiceException e) {
          throw new OperatorFailedException("NetServiceException: Konnte geratene URL "+guessedURL+" nicht "+
                                            "holen.");
      }

      LogService.logPlan(this, "Geratene URL "+guessedURL+" konnte geholt werden.");      

      //Erzeuge potentielle compage-Instanz
      Instance nri = new Instance(guessedURL);
      nri.setNetEntity(nr);
    
      //Das spezifische tradepage-Unterkonzept zur trade-Instanz holen
      PredArgument tradePA = this.precond[0].getPredArg(1);
      String tradeAsString = tradePA.getInstance().name;
      T_Box tbox = tradePA.getConcept().getT_Box();
      Concept specTradeC = tbox.lookup(tradeAsString);
    
      //potentielle compage-Instanz bzgl. spezifischen tradepage-Unterkonzept klassifizieren
      if(specTradeC.classify(nri)) {
          //...positive Klassifikation

	  double classVal = 0.01; //Initialisierung
	  
	  try {
	      classVal = nr.getClassification(specTradeC.name);
	  }
	  catch(NetResultNoClassificationException e) {
	      LogService.logPlan(this, "Error! Trotz positiver Klassifikation steigt nr.getClassification"+
				 " mit NetResultNoClassificationException aus!");
	  }
	  LogService.logPlan(this, "Positive Klassifikation von "+guessedURL+" nach "+specTradeC.name+" ("+classVal+") => postcondFullfilled=1.");


	  //has_compage Predicate und seine PredArgs bestimmen
	  Predicate has_compageP = (Predicate) (this.getPostcond())[1].getAddList().getFirst();
	  PredArgument companynamePA = has_compageP.getPredArg(0);
          PredArgument compagePA = has_compageP.getPredArg(1);

	  //Neue Instanz in compage PredArg einsetzen
          compagePA.setInstance(nri);

	  //Der Konfidenzwert von has_compage ergibt sich als Produkt
	  //der Konfidenz der companyname-Instanz und dem Klassifikationswert fuer tradepage.
	  //Bei vorgegebenen companyname ist dies also der tradepage-Klassifikationswert.

	  double companynameConf = companynamePA.getInstance().getConfidence(companynamePA.getConcept());
	  double confidence = classVal * companynameConf;
	  has_compageP.setConfidence(confidence);
	  LogService.logPlan(this, "Konfidenz von has_compage: "+classVal+"*"+companynameConf+"="+confidence);
     
	  //Erfuellte Nachbedingung markieren
          this.postcondFullfilled=1;

      } else {
          //...negative Klassifikation   
          LogService.logPlan(this, "Negative Klassifikation von "+guessedURL+" nach "+specTradeC.name+" => postcondFullfilled=0.");

	  //Erfuellte Nachbedingung markieren
          this.postcondFullfilled=0;

      };//else

  }//eval


    /**
        Abstrakte Methode, in der die compage URL geraten wird. Genau diese Methode muessen
        abgeleitete Klassen ausprogrammieren.
     */
    protected abstract String guessURL();


  public GuessCompageOperator(){
    /* 1 Vorbedingung(en) */
    this.precond=new Predicate[1];
    /* Vorbedindungsprdikat 0 */
    this.precond[0]=new Predicate(2,"in_trade");

    PredArgument comname = new PredArgument("companyname");
    comname.setCreatesWorldObject(false);
    this.precond[0].setPredArg(comname, 0);
    PredArgument trade = new PredArgument("trade");
    trade.setCreatesWorldObject(false);
    this.precond[0].setPredArg(trade, 1);

    Predicate pred;
    /* 2 Nachbedingung(en) */
    this.postconds=new Postcondition[2];

    /* Nachbedingung 0 */
    this.postconds[0]=new Postcondition();

    /* Nachbedingung 1 */
    this.postconds[1]=new Postcondition();
    /* AddList */
    pred=new Predicate(2,"has_compage");
    pred.setPredArg(comname, 0);
    PredArgument compage = new PredArgument("compage");
    compage.setCreatesWorldObject(true);
    pred.setPredArg(compage, 1);

    this.postconds[1].addAddListPred(pred);

    /* DeleteList */
  }//Konstruktor

}//class GuessCompageOperator




