package operators;
import informationexchange.LogService;
import knowledge.*;
import java.util.Iterator;
import exceptions.*;
import java.util.LinkedList;
import dataprovider.net.*;
import dataprovider.enumerationobjects.*;
import dataanalysis.tools.StringMatcher;

/**
   automatisch erzeugter Operator GuessCompanynameOperator
   @author Oliver Geppert
   @version 1.0 [1999-12-10]
*/
public class GuessCompanynameOperator extends Operator{

  /**
    Konstruktor fr den GuessCompanynameOperator. 
        Vorbedingung: contains_prodname(prodpage,prodname)
    1. Nachbedingung: *Leer*
    2. Nachbedingung: add has_productpage(comname,prodpage):y,n
                      add deals_in(comname,prodname):n,n	
    @since [1999-12-10]
   */
  public GuessCompanynameOperator(){
    /* 1 Vorbedingung(en) */
    precond=new Predicate[1];
    /* Vorbedindungsprdikat 0 */
    precond[0]=new Predicate(2,"contains_productname");

    PredArgument productpage = new PredArgument("productpage");
    productpage.setCreatesWorldObject(false);
    precond[0].setPredArg(productpage, 0);
    PredArgument prodname = new PredArgument("prodname");
    prodname.setCreatesWorldObject(false);
    precond[0].setPredArg(prodname, 1);

    Predicate pred;
    /* 2 Nachbedingung(en) */
    postconds=new Postcondition[2];

    /* Nachbedingung 0 */
    postconds[0]=new Postcondition();
    /* AddList */
    /* DeleteList */
    /* Nachbedingung 1 */
    postconds[1]=new Postcondition();
    /* AddList */
    pred=new Predicate(2,"has_productpage");
    PredArgument companyname = new PredArgument("companyname");
    companyname.setCreatesWorldObject(true);
    pred.setPredArg(companyname, 0);
    pred.setPredArg(productpage, 1);


    postconds[1].addAddListPred(pred);

    pred=new Predicate(2,"deals_in");
    pred.setPredArg(companyname, 0);
    pred.setPredArg(prodname, 1);


    postconds[1].addAddListPred(pred);

    /* DeleteList */
  }
  /**
     Eval-Methode des GuessCompanynameOperators.
     Naeheres siehe {@link operators.Operator#eval()}.
     Die eval-Methode hat die Aufgabe, mit Hilfe einer Productpage
     und eines Productnames einen Companyname zu erzeugen. Als Heuristik
     wird die URL der "ubergebenen Seite so auseinandergenommen, da"s aus
     z.B. http//:www.dortmund.de dortmund wird, aus http://dortmund.com
     auch. Als Einschr"ankung wird verlangt, da"s der so gefundene
     GuessedComname im Text der Seite mindestens einmal vorkommen mu"s.
     Die URLs von Links auf der Seite werden durch getText() ingoriert, so
     da"s der GuessedComname im Flie"stext vorkommen mu"s.
     @since [1999-12-10]
  */
  public void eval(){

    postcondFullfilled = 0;
  
    String GuessedComname = "";

    PredArgument prodpage = getPrecond()[0].getPredArg(0);
    PredArgument prodname = getPrecond()[0].getPredArg(1);
    Predicate has_productpage = (Predicate) getPostcond()[1].getAddList().get(0);
    PredArgument comname =
                          ((Predicate) getPostcond()[1].getAddList().get(0)).
                          getPredArg(0);
    try {

      NetEntity ProdNE = prodpage.getInstance().getNetEntity();
      HTMLTextNetResult ProdHTML = ProdNE instanceof HTMLTextNetResult ?
                                   (HTMLTextNetResult) ProdNE
                                   : null;
      if (ProdHTML != null) {
        String ProdURL = ProdHTML.getURL ();
        String ProdText = ProdHTML.getText ();
        String Hostname = StringUtils.extractURLsProtAndHost(ProdURL);
        if (Hostname.startsWith("http://")) 
                   Hostname.substring(7, Hostname.length());
        GuessedComname = Hostname;
        int fi;
        int li;
        if ((fi = Hostname.indexOf(".")) < (li = Hostname.lastIndexOf(".")))
	    GuessedComname = Hostname.substring(fi+1, li);/* Der Endindex wird nicht
							     mehr beruecksichtigt
							     von der Methode.
							     Der letzte Punkt faellt
							     also eh weg */ //Aenderung NM
        if (fi == li && fi != -1) 
           GuessedComname = Hostname.substring(0, fi);
        StringMatcher sm = new StringMatcher (GuessedComname);
        if (sm.fastMatch(ProdText) == 0) 
          GuessedComname = "";
        
      }

    } catch (NetResultReloadFailedException e) {
      LogService.logException (this, "NetResultReloadFailedException");
      GuessedComname = "";
    } catch (java.net.MalformedURLException e) {
      LogService.logException (this, "MalformedURLException");
      GuessedComname = "";
    }

    if (!GuessedComname.equals("")) {
	//Aenderung Christian
	Instance comnameI = new Instance(GuessedComname);
	comnameI.addConcept(comname.getConcept(), 0.9);
        comname.setInstance(comnameI);
        has_productpage.setConfidence (1.0);

        postcondFullfilled = 1;
    }
  }

}




